#!/usr/bin/python3

import re
import urllib3
import json
from urllib.parse import urlencode
from screenlogger.screenlogger import Msg
from simplesteem.simplesteem import SimpleSteem
from steemax import axdb
from steemax import default
from steemax import sec


class Web:
    def __init__ (self):
        self.steem = SimpleSteem(
                client_id=default.client_id,
                client_secret=default.client_secret,
                callback_url=default.callback_url,
                screenmode=default.msgmode)
        self.db = axdb.AXdb(
                default.dbuser, 
                default.dbpass, 
                default.dbname)
        self.msg = Msg(default.logfilename, 
                default.logpath,
                default.msgmode)


    def load_template(self, templatefile=""):
        ''' opens a template file and loads it
        into memory stored in a variable
        '''
        templatepath = default.webpath + "/" + templatefile            
        with open(templatepath, 'r') as fh:
            try:
                template = fh.read()
            except Exception as e:
                self.msg.error_message(e)
                return None
            else:
                return template


    def make_page(self, template, **kwargs):
        ''' Fills in key / value pairs on a 
        given template
        '''
        regobj = re.compile(
            r"^(.+)(?:\n|\r\n?)((?:(?:\n|\r\n?).+)+)", 
            re.MULTILINE)
        newtemplate = regobj.sub('', template)
        for key, value in kwargs.items():
            newtemplate = re.sub(str(key), 
                                str(value), 
                                newtemplate)
        return newtemplate


    def login(self, token, dest="home"):
        ''' logs a user in using SteemConnect
        adds the user to the database if it's
        their first time.
        '''
        if self.verify_token(token):
            if self.db.get_user_token(self.steem.username):
                self.db.update_token(self.steem.username, 
                        self.steem.accesstoken, 
                        self.steem.refreshtoken)
            else:
                self.db.add_user(self.steem.username, 
                        self.steem.privatekey, 
                        self.steem.refreshtoken, 
                        self.steem.accesstoken)
            if dest == "info":
                return ("\r\n" 
                        + self.info_page(self.steem.username))
            else:
                return ("\r\n" 
                        + self.make_page(self.load_template(
                        "templates/index.html"), 
                        ACCOUNT1=self.steem.username,
                        REFRESHTOKEN=self.steem.refreshtoken))
        else:
            return self.auth_url()


    def invite(self, token, account2, per, 
                    ratio, dur, response, ip):
        ''' Creates an invite
        '''
        response = sec.filter_token(response)
        if not self.verify_recaptcha(response, ip):
            return self.error_page("Invalid captcha.")
        if self.verify_token(sec.filter_token(token)):
            account2 = sec.filter_account(account2)
            if self.steem.account(account2) == False:
                return self.error_page(account2 + " is an invalid account name.")
            else:
                memoid = self.db.add_invite(
                        self.steem.username, 
                        account2,  
                        sec.filter_number(per), 
                        sec.filter_number(ratio, 1000), 
                        sec.filter_number(dur, 365))
                if memoid == False:
                    return self.error_page(self.db.errmsg)
                elif int(memoid) > 0:
                    return ("Location: https://steemax.info/@" 
                            + self.steem.username + "\r\n")
        else:
            return self.auth_url()


    def archive_page(self, account=None):
        ''' Provides a list of all the exchanges that
        have occured for a particular account. if
        no account is provided then a list of all exchanges
        for all accounts is returned.
        '''
        infobox = ""
        if account is not None:
            account = sec.filter_account(account)
            if not self.db.get_user_token(account):
                return self.auth_url()
        axlist = self.db.get_exchange_archive(account)
        boxtemplate = self.load_template("templates/archivebox.html")
        for trade in axlist:
            box = self.make_page(boxtemplate,
                            TIMESTAMP=trade[6],
                            ACCOUNT1=trade[0],
                            ACCOUNT2=trade[1],
                            VOTEVALUE1=trade[4],
                            VOTEVALUE2=trade[5])
            infobox = infobox + box
        pagetemplate = self.load_template("templates/archive.html")
        return ("\r\n" + self.make_page(pagetemplate,
                                ACCOUNT1=account,
                                INFOBOX=infobox))


    def info_page(self, account):
        ''' Creates the page at steemax.info that displays
        all the exchanges a user is involved in and which
        provides the options to accept, barter or cancel a 
        particular exchange.
        '''
        account = sec.filter_account(account)
        if not self.db.get_user_token(account):
            return self.auth_url()
        axlist = self.db.get_axlist(account)
        boxtemplate = self.load_template("templates/infobox.html")
        infobox = ""
        invitee = 0
        otheraccount = ""
        buttoncode = ""
        exp = ""
        for value in axlist:
            if account == value[2]:
                invitee = 1
                otheraccount = value[1]
            else:
                invitee = 0
                otheraccount = value[2]
            if int(value[7]) == -1 and invitee == 0:
                buttoncode = '''
                <div class="button" onClick="command('{}', '{}', '{}')">Start</div>
                                '''.format(
                                value[0], 
                                otheraccount,
                                "start")
            if ((int(value[7]) == 0 and invitee == 1) 
                    or (int(value[7]) == 2 and invitee == 1)
                    or (int(value[7]) == 3 and invitee == 0)):
                buttoncode = '''
                <div class="button" onClick="command('{}', '{}', '{}')">Accept</div>
                <div class="button" onClick="barter_window('{}');">Barter</div>
                                '''.format(
                                value[0],
                                otheraccount,
                                "accept",
                                value[0])
            elif ((int(value[7]) == 0 and invitee == 0)  
                    or (int(value[7]) == 3 and invitee == 1)
                    or (int(value[7]) == 2 and invitee == 0)):
                buttoncode = '<div id="pending">Pending</div>\n'
            if int(value[7]) == 1:
                buttoncode = '<div id="pending">Active</div>\n'
                exp = "Expires " + self.db.expiration_date(value[8], value[5])
            else:
                exp = value[5] + " days"
            buttoncode = buttoncode + '''
                <div class="button" onClick="command('{}', '{}', '{}')">
                                Cancel</div>
                                '''.format(
                                value[0],
                                otheraccount,
                                "cancel")
            if int(value[7]) != 4 and not (int(value[7]) == -1 and invitee == 1):
                box = self.make_page(boxtemplate,
                                AXID=value[0],
                                ACCOUNT1=value[1],
                                ACCOUNT2=value[2],
                                PERCENTAGE=value[3],
                                DURATION=exp,
                                DARATIO=value[4],
                                MEMOID=value[6],
                                INVITEE=invitee,
                                OTHERACCOUNT=otheraccount,
                                BTNCODE=buttoncode)
                infobox = infobox + box
        pagetemplate = self.load_template("templates/info.html")
        return ("\r\n" + self.make_page(pagetemplate,
                                ACCOUNT1=account,
                                INFOBOX=infobox))


    def verify_token(self, token):
        ''' cleans and verifies a SteemConnect
        refresh token
        '''
        token = sec.filter_token(token)
        if (token is not None
                    and self.steem.verify_key(
                    acctname="", tokenkey=token)):
            return True
        else:
            return False


    def auth_url(self):
        ''' Returns the SteemConnect authorization
        URL for SteemAX
        '''
        url = self.steem.connect.auth_url()
        return ("Location: " + url + "\r\n")


    def error_page(self, msg):
        ''' Returns the HTML page with the
        given error message
        '''
        return ("\r\n" 
                + self.make_page(
                self.load_template("templates/error.html"), 
                ERRORMSG=msg))


    def verify_recaptcha(self, response, remoteip):
        ''' Verifies a Google recaptcha v2 token
        '''
        http = urllib3.PoolManager()
        encoded_args = urlencode({'secret': default.recaptcha_secret,
                            'response': response,
                            'remoteip': remoteip})
        url = default.recaptcha_url + "?" + encoded_args
        req = http.request('POST', url)
        if req.status == 200:
            self.json_resp = json.loads(req.data.decode('utf-8'))
        if self.json_resp['success']:
            return True
        else:
            return False


# EOF



#!/usr/bin/python3

from steemax import axdb
from steemax import axverify
from steemax import default

class Axe:

    def __init__(self):
        self.db = axdb.AXdb(default.dbuser, 
                            default.dbpass, 
                            default.dbname)
        self.verify = axverify.AXverify()


    def exchange(self):
        ''' This method actualizes the exchange between Steemians.
        ID          row[0]
        Inviter     row[1]
        Invitee     row[2]
        Percentage  row[3]
        Ratio       row[4]
        Duration    row[5]
        MemoID      row[6]
        Status      row[7]
        Timestamp   row[8]
        '''
        axlist = self.db.get_axlist(run=True)
        for row in axlist:
            print("Comparing "+str(row[1])+" vs. "+str(row[2]))
            if self.verify.eligible_posts(row[1], row[2]) is not False:
                print("Posts are eligible.")
                if self.verify.eligible_votes(row[1], 
                                    row[2], 
                                    row[3], 
                                    row[4], 
                                    2) is not False:
                    print("Votes are eligible")
                    # Inviter votes invitee's post
                    self.verify.vote_on_it(row[1], 
                                row[2], 
                                self.verify.post_two, 
                                int(row[3]))
                    # Invitee votes inviter's post
                    self.verify.vote_on_it(row[2], 
                                row[1], 
                                self.verify.post_one, 
                                self.verify.vote_cut)
                    self.verify.get_vote_value(row[1], int(row[3]))
                    vv1 = self.verify.votevalue
                    self.verify.get_vote_value(row[2], self.verify.vote_cut)
                    vv2 = self.verify.votevalue
                    self.db.archive_exchange(row[6], row[1], row[2], 
                                                vv1, vv2, 
                                                self.verify.post_one, 
                                                self.verify.post_two)



# EOF



#!/usr/bin/python3

from screenlogger.screenlogger import Msg
from simplesteem.simplesteem import SimpleSteem
from steemax import default
from steemax import axdb


class AXverify:


    def __init__(self):
        self.steem = SimpleSteem(client_id=default.client_id,
                                client_secret=default.client_secret,
                                callback_url=default.callback_url,
                                screenmode=default.msgmode)
        self.msg = Msg(default.logfilename, 
                        default.logpath,
                        default.msgmode)
        self.response = None
        self.post_one = ""
        self.post_two = ""
        self.vote_cut = 0


    def vote_on_it(self, voter, author, post, weight):
        ''' Refresh the token in the database
        and use voter's token to upvote 
        author's post
        '''
        db = axdb.AXdb(default.dbuser, 
                        default.dbpass, 
                        default.dbname)
        if db.get_user_token(voter) is not False:
            accesstoken = db.dbresults[0][1]
            refreshtoken = db.dbresults[0][2]
        else:
            return False
        if not self.sc_vote(voter, author, post, weight, accesstoken):
            print("Renewing token for " + voter)
            newtoken = self.renew_token(voter, refreshtoken)
            if newtoken is not False:
                return self.sc_vote(voter, author, post, weight, newtoken)
            else:
                self.msg.error_message("A NEW TOKEN COULD NOT BE CREATED")
                return False


    def sc_vote(self, voter, author, post, weight, token):
        self.steem.connect.sc = None
        self.steem.connect.steemconnect(
                        token)
        result = self.steem.connect.vote(
                        voter, 
                        author, 
                        post, 
                        int(weight))
        try:
            result['error']
        except:
            # The vote was successful
            print(str(voter)+" has voted on "
                            +str(post)+" "
                            +str(weight)+"%")
            return True 
        else:
            self.msg.error_message(str(result)) 
            return False
        

    def renew_token(self, accountname, refreshtoken):
        ''' If the access token has expired
        use the refresh token to get a new accss token.
        '''
        if self.steem.verify_key(
                        acctname="", tokenkey=refreshtoken):
            db.update_token(self.steem.username,
                    self.steem.accesstoken, 
                    self.steem.refreshtoken)
            return self.steem.accesstoken
        else:
            return False


    def get_vote_value (self, acctname, voteweight=100, 
                        votepower=0):
        ''' Voteweight and votepower are entered 
        as a percentage value (1 to 100)
        If the default is used for votepower (0) 
        it is set by the system at the time
        the account last voted.
        '''
        self.steem.check_balances(acctname)
        if votepower == 0:
            votepower = self.steem.votepower
        self.votevalue = self.steem.current_vote_value(
                    lastvotetime=self.steem.lastvotetime, 
                    steempower=self.steem.steempower, 
                    voteweight=int(voteweight), 
                    votepower=int(votepower))
        self.voteweight = voteweight
        self.votepower = votepower
        return self.steem.rshares


    def verify_post (self, account1, account2):
        ''' Gets only the most recent post, gets the 
        timestamp and finds the age of the post in days.
        Is the post too old? Did account2 
        vote on the post already?
        '''
        identifier = self.steem.recent_post(account1)
        if identifier is False or identifier is None:
            self.msg.error_message("No post for " + account1)
            return False
        else:
            permlink = self.steem.util.permlink(identifier)
            votes = self.steem.vote_history(permlink[1], account1)
            for v in votes:
                if v['voter'] == account2:
                    self.msg.message(account2 + 
                        " has aready voted on " + permlink[1])
                    return False
        return permlink[1]


    def eligible_posts (self, account1, account2):
        ''' Verify the posts of both accounts
        '''
        post = self.verify_post(account1, account2)
        if post is False or post is None:
            self.msg.message(account1 
                + " does not have an eligible post.")
            return False
        else:
            self.post_one = post
        post = self.verify_post(account2, account1)
        if post is False or post is None:
            self.msg.message(account2 
                + " does not have an eligible post.")
            return False
        else:
            self.post_two = post
        return True


    def eligible_votes(self, account1, account2, 
                        percentage, ratio, flag):
        ''' If the flag is raised use the full voting 
        power to make the comparison rather
        than the current voting power. 
        If ratio was not acheived return false 
        Otherwise display approximate 
        upvote matches
        '''
        if flag == 1:
            vpow = 100
        else:
            vpow = 0
        v1 = self.get_vote_value(account1, percentage, vpow)
        v2 = self.get_vote_value(account2, 100, vpow)
        v3 = ((v1 / v2) * 100) / float(ratio)
        v3a = round(v3, 2)
        exceeds = False
        if v3a < 1:
            v3 = 1
            exceeds = True
        if v3a > 100:
            v3 = 100
            exceeds = True
        self.msg.message(account2 + " needs to vote " 
                + str(v3a) + "% in order to meet " + account1)
        self.vote_cut = v3
        v4 = self.get_vote_value(account2, v3, vpow)
        v1s = self.steem.rshares_to_steem(v1)
        v4s = self.steem.rshares_to_steem(v4)
        if exceeds and flag != 2:
            if v3 == 1:
                v5 = v4 - v1
                v5s = self.steem.rshares_to_steem(v5)
                self.response = (account2 + "'s vote of " + str(v4s) 
                                + " will be larger than " + account1 
                                + "'s vote by: " + str(v5s))
                self.msg.message(self.response)
            if v3 == 100:
                v5 = v1 - v4
                v5s = self.steem.rshares_to_steem(v5)
                self.response = (account1 + "'s vote of " + str(v1s) 
                                + " will be larger than " + account2 
                                + "'s vote by: " + str(v5s))
                self.msg.message(self.response)
            return False
        else:
            self.msg.message(account1 + " will upvote $" + str(v1s) 
                    + " and " + account2 + " will upvote $" + str(v4s))
            return True


# EOF



#!/usr/bin/python3

import pymysql
import re
from screenlogger.screenlogger import Msg
from steemax import default

class DB():


    def __init__(self, dbuser, dbpass, dbname):
        self.dbuser = dbuser
        self.dbpass = dbpass
        self.dbname = dbname
        self.msg = Msg(default.logfilename, 
                        default.logpath, 
                        default.msgmode)


    def open_db(self):
        ''' opens a database connection
        '''
        self.db = pymysql.connect("localhost",
                                    self.dbuser,
                                    self.dbpass,
                                    self.dbname)
        self.cursor = self.db.cursor()


    def get_results(self, sql, *args):
        ''' Gets the results of an SQL statement
        '''
        self.open_db()
        try:
            self.cursor.execute(pymysql.escape_string(sql), args)
            self.dbresults = self.cursor.fetchall()
        except Exception as e:
            self.msg.error_message(e)
            self.dbresults = False
            self.db.rollback()
            return False
        else:
            return len(self.dbresults)
        finally:
            self.db.close()


    def commit(self, sql, *args):
        ''' Commits the actions of an SQL 
        statement to the database
        '''
        self.open_db()
        try:
            self.cursor.execute(pymysql.escape_string(sql), args)
            self.db.commit()
        except Exception as e:
            self.msg.error_message(e)
            self.db.rollback()
            return False
        else:
            return True
        finally:
            self.db.close()


# EOF



#!/usr/bin/python3

import sys
import re
from dateutil.parser import parse
from datetime import datetime
from steemax import axdb
from steemax import axverify
from steemax import default
from steemax import sec
from screenlogger.screenlogger import Msg
from simplesteem.simplesteem import SimpleSteem


class MemoMsg():


    def __init__(self):
        self.db = axdb.AXdb(default.dbuser, 
                            default.dbpass, 
                            default.dbname)


    def invite_msg(self, acct1, acct2, 
                memoid):
        ''' Creates the invite message
        '''
        self.reaction = "started"
        self.db.update_status(0, memoid)
        invite = self.db.get_invite(memoid)
        msg = ("Hello @{}, @{} has invited you "
            + " to an auto-upvote exchange. "
            + "They have offered {}% of their upvote "
            + "At a ratio of {}:1 for {} days. "
            ).format(acct2, acct1, 
            invite[0], invite[1], invite[2])
        if self.db.get_user_token(acct2):
            msg = msg + ("Please send any amount SBD to "
                    + "@steem-ax with the following in "
                    + "the memo: '{}:accept'. "
                    ).format(memoid)
        msg = msg + ("For more info please visit "
                    + "https://steemax.info/@{}"
                    ).format(acct2)
        self.returnmsg = (msg)


    def accepted_msg(self, acct, memoid):
        ''' Creates the accept message 
        '''
        self.reaction = "accepted"
        self.db.update_status(1, memoid)
        self.returnmsg = (
            "@{} has accepted the invite. ".format(acct)
            + "The auto-upvote exchange will "
            + "begin immediately.")


    def barter_msg(self, reaction, status, acct, 
                    per, ratio, dur, memoid, acct2):
        ''' Creates the barter message
        '''
        self.reaction = reaction
        if status == 2:
            pronoun = "their"
        else:
            pronoun = "your"
        self.db.update_invite(per, ratio, 
                            dur, memoid, status)
        self.returnmsg = (
            ("@{} has offered to barter. They offer " 
            + "{}% of {} upvote at a ratio of " 
            + "{}:1 for {} days. " 
            + "Send any SBD along with this " 
            + "memo code to @steem-ax to accept this invite: "
            + "\n\n'{}:accept'. " 
            + "To generate your own barter offer, "
            + "please visit https://steemax.info/@{}").format(
            acct, per, pronoun, ratio, dur, memoid, acct2))


    def ignore(self, reason):
        ''' Creates the ignore message 
        '''
        self.reaction = "refund"
        self.returnmsg = reason


    def cancel(self, canceller, memoid):
        ''' The cancel command can be 
        given at any time by either party
        '''
        self.reaction = "cancelled"
        self.db.update_status(4, memoid)
        self.returnmsg = ("@" + canceller 
            + " has cancelled the exchange.")


class Reaction(MemoMsg):


    def start(self, acct1, acct2, memofrom, 
                    rstatus, memoid):
        ''' The start method grants the authority to the 
        exchange from the inviter. 
        '''
        if acct1 == memofrom:
            if rstatus < 1:
                self.invite_msg(acct1, acct2, memoid)
            else:
                self.ignore("@{} has already".format(acct1)
                        + "started this exchange.")
        else:
            self.ignore("Invalid action. @{} ".format(acct1)
                        + " has not yet started this "
                        + "exchange.")


    def accept(self, acct1, acct2, memofrom, 
                    rstatus, memoid):
        ''' The accept method is used to authorize 
        a change whenever an invite or a barter is 
        sent. 
        '''
        invalid = False
        if acct1 == memofrom:  
            if (int(rstatus) > -1 and int(rstatus) != 2):
                self.accepted_msg(acct1, memoid)
        elif acct2 == memofrom:
            if (int(rstatus) > -1 and int(rstatus) < 3):
                self.accepted_msg(acct2, memoid)
        else:
            self.ignore("Invalid Memo ID.")


    def barter(self, acct1, acct2, memoid, memofrom, 
                    rstatus, per, ratio, dur):
        ''' A barter command can be used any 
        time after both parties have authorized 
        the exchange.
         2 = inviter (account1) has offered to barter
         3 = invitee (account2) has offered to barter
        '''
        invalid = False
        verify = axverify.AXverify()
        if not verify.eligible_votes(
                    acct1, acct2, per, ratio, 1):
            self.ignore("Barter invalid. " + verify.response)
            return
        if acct1 == memofrom:
            if (int(rstatus) > -1 and int(rstatus) != 2):
                self.barter_msg("acct1 bartering", 2, 
                                acct1, per, ratio, 
                                dur, memoid, acct2)
            else:
                invalid = True
        elif acct2 == memofrom:
            if (int(rstatus) > -1 and int(rstatus) < 3):
                self.barter_msg("acct2 bartering", 3, 
                                acct2, per, ratio, 
                                dur, memoid, acct1)
            else:
                invalid = True
        else:
            self.ignore("Invalid Memo ID.")
        if invalid:
            self.ignore("Invalid barter. Please be sure that @"
                        + acct1 + " has started the invite. "
                        + "If you've already sent a barter "
                        + "please wait for the other side to "
                        + "respond.")


class AXtrans:


    def __init__(self):
        self.msg = Msg(default.logfilename, 
                        default.logpath, 
                        default.msgmode)
        self.db = axdb.AXdb(default.dbuser, 
                            default.dbpass, 
                            default.dbname)
        self.steem = SimpleSteem()
        self.react = Reaction()


    def parse_memo(self, **kwargs):
        ''' Parses the memo message in a transaction
        for the appropriate action.
        '''
        for key, value in kwargs.items():
            setattr(self, key, value)
        try:
            memo = self.memo.split(":")
        except:
            return False
        self.memoid = sec.filter_token(memo[0])
        if len(memo) == 2:
            self.action = sec.filter_account(memo[1])
            return True
        elif len(memo) == 5:
            self.action = sec.filter_account(memo[1])
            self.percentage = sec.filter_number(memo[2])
            self.ratio = sec.filter_number(memo[3], 1000)
            self.duration = sec.filter_number(memo[4], 365)
            return True
        else:
            return False


    def send(self, to="artopium", 
                    amt="0.001 SBD", 
                    msg="test"):
        ''' Sends the forwarded amount of SBD along
        with the reaction message
        '''
        r = amt.split(" ")
        if self.steem.transfer_funds(to, float(r[0]), 
                                    r[1], msg):
            self.msg.message(("Transaction committed. Sent {} "
                            "{} to {} with the memo: "
                            "{}").format(r[0], r[1], to, msg))


    def act(self, acct1, acct2, rstatus, sendto):
        ''' Decides how to react baed on the action
        present in the memo message
        '''
        if not self.db.get_user_token(self.memofrom):
            self.react.ignore(
                ("@{} is not a current member of "
                + " https://steemax.trade !!Join now!! "
                + "using SteemConnect.").format(self.memofrom))
        elif (self.action == "start"):
            self.react.start(acct1, 
                            acct2, 
                            self.memofrom, 
                            rstatus,
                            self.memoid)
        elif (self.action == "cancel"):
            self.react.cancel(self.memofrom, 
                            self.memoid)
        elif (self.action == "accept"):
            self.react.accept(acct1, 
                            acct2, 
                            self.memofrom, 
                            rstatus,
                            self.memoid)
        elif (self.action == "barter"):
            self.react.barter(acct1, 
                            acct2, 
                            self.memoid, 
                            self.memofrom, 
                            rstatus, 
                            self.percentage, 
                            self.ratio, 
                            self.duration)
        else:
            self.react.ignore("Invalid action.")
        if self.react.reaction == "refund":
            self.sendto = self.memofrom
        else:
            self.sendto = sendto


    def parse_history_record(self, record, lasttrans):
        ''' Parses the blockchain record for transdactions
        sent to @steem-ax 
        '''
        if (record[1]['op'][0] == 'transfer'
                and datetime.strptime(
                    record[1]['timestamp'], 
                    '%Y-%m-%dT%H:%M:%S')  
                > lasttrans
                and re.match(r'^[0-9]{32}', 
                    record[1]['op'][1]['memo'])
                and record[1]['op'][1]['to'] 
                == self.steem.mainaccount):
            return True
        else:
            return False


    def fetch_history(self):
        ''' Processes the transaction history. 
        The memo message is parsed for a valid 
        command and performs the 
        directed action.
        '''
        rt = self.db.get_most_recent_trans()
        for a in self.steem.get_my_history():
            if self.parse_history_record(a, rt):
                if self.parse_memo(memofrom=a[1]['op'][1]['from'], 
                                memo=a[1]['op'][1]['memo'], 
                                amount=a[1]['op'][1]['amount'], 
                                trxid=a[1]['trx_id'], 
                                txtime=a[1]['timestamp']):
                    if self.db.verify_memoid(self.memofrom, self.memoid):
                        self.act(self.db.dbresults[0][0],
                                self.db.dbresults[0][1],
                                int(self.db.dbresults[0][2]),
                                self.db.sendto)
                    else:
                        self.react.ignore("Invalid Memo ID.")
                        self.sendto = self.memofrom
                else:
                    self.react.ignore("Invalid Memo.")
                    self.sendto = self.memofrom
                self.send(self.sendto, 
                                self.amount, 
                                self.react.returnmsg)
                self.db.add_trans(self.trxid, 
                                self.memofrom, 
                                self.amount, 
                                self.memoid, 
                                self.react.reaction, 
                                self.txtime)


# EOF



#!/usr/bin/python3

import re


def filter_token (string):
    string = re.sub(r'[^A-Za-z0-9\-\_\.]', 
                '', str(string))
    string = string[0:500]
    if string is None:
        string = "0"
    return string


def filter_account (string):
    string = re.sub(r'[^A-Za-z0-9\-\_\.]', 
                '', str(string))
    string = string[0:32]
    if string is None:
        string = " "
    return string


def filter_number (string, limit=100):
    string = re.sub(r'[^0-9\.]', 
                '', str(string))
    string = string[0:6]
    if float(string) < 0.001:
        string = "1"
    if float(string) > limit:
        string = limit
    return string
